@@ -1,5 +1,5 @@
 # dictformat-cannadic.rb: Converter module for cannadic format.
-# $Id: dictformat.rb,v 1.1.2.2 2003/12/29 09:50:35 komatsu Exp $
+# $Id: dictformat.rb,v 1.2 2004/03/25 17:19:51 komatsu Exp $
 #
 # Copyright (C) 2003 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -54,7 +54,11 @@ class DictFormat
   ## This method should be over-written.
   def parse (line)
     line.chomp!
-    return [line.split(/\t/)]
+    if line !~ /\t/ then
+      return []
+    else
+      return [line.split(/\t/)]
+    end
   end
 
   def sort_result (results)
@@ -100,7 +104,7 @@ class DictFormat
     pos = (pos1 or pos2)
 
     freq = (freq1.to_i or 0)
-    attr = attr2
+    attr = attr2 + attr1
     if pos1 == pos2 or pos1.nil? or pos2.nil? then
       freq  = [freq, freq2].max
       attr += attr2
