@@ -1,5 +1,5 @@
 # prime/prime.rb
-# $Id: prime.rb,v 1.2.2.21 2004/02/08 11:58:07 komatsu Exp $
+# $Id: prime.rb,v 1.7 2004/03/25 17:19:49 komatsu Exp $
 #
 # Copyright (C) 2002, 2003, 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -121,7 +121,7 @@ class Prime
   end
 
   def preedit_convert_input (pattern)
-    return PrimeTypeConv::convert(pattern).join("\t")
+    return PrimeTypeConv::convert(pattern).join()
   end
 
   def set_context (context)
@@ -145,62 +145,90 @@ class Prime
         PRIME_ENV['typing_method'] == 'handwrite' then
       word_list = lookup_direct(string)
     else
+#      word_list = lookup_compact(string)
       word_list = lookup_hybrid(string)
     end
     return word_list
   end
+  def lookup_all (string)
+    if PRIME_ENV['typing_method'] == 'tcode' or 
+        PRIME_ENV['typing_method'] == 'handwrite' then
+      word_list = lookup_direct_all(string)
+    else
+      word_list = lookup_hybrid_all(string)
+    end
+    return word_list
+  end
 
   def lookup_hybrid (string)
-    results_prefix   = lookup_prefix(string)
-    results_japanese = lookup_japanese(string)
-    results_overall  = lookup_overall(string)
+    if @cache_hybrid_pattern == string then
+      return @cache_hybrid
+    end
 
-    ## FIXME: Delete the Magic Numbers.
-    prefix_length  = 10 - [results_japanese.length, 7].min
-    results = PrimeWordList::merge_with_label(@context,
-                                              results_prefix[0,prefix_length],
-                                              results_overall,
-                                              results_japanese)
-    return results
+    words  = search_hybrid(string, 10)
+    result = PrimeWordList::merge_with_label(@context, words)
+
+    @cache_hybrid_pattern = string
+    @cache_hybrid = result
+    return result
+  end
+  def lookup_hybrid_all (string)
+    words  = search_hybrid(string)
+    result = PrimeWordList::merge_with_label(@context, words)
+    return result
   end
 
   def lookup_direct (string)
+    results_expansion = lookup_expansion(string)
+    results = PrimeWordList::merge_with_label(@context, results_expansion)
+    return results[0,3]
+  end
+
+  def lookup_direct_all (string)
     #     results_prefix    = lookup_prefix(string)
     results_expansion = lookup_expansion(string)
     results_mixed     = lookup_mixed(string)
-    results_japanese  = lookup_japanese(string)
-    results_overall   = lookup_overall(string)
+    words_japanese  = search_japanese(string)
+    words_overall   = search_overall(string)
 
     results = PrimeWordList::merge_with_label(@context,
                                               # results_prefix[0, 3],
                                               results_expansion,
                                               results_mixed,
-                                              results_japanese,
-                                              results_overall)
-    return results
+                                              words_japanese,
+                                              words_overall)
+    return PrimeWordList.new(results_expansion[0,3] | results)
+  end
+
+  def lookup_compact (string)
+    words_compact = search_compact(string)
+    words = PrimeWordList::merge_with_label(@context, words_compact)
+    return words
+  end
+  def lookup_compact_all (string)
+    words_compact  = search_compact(string)
+    words_overall  = search_overall(string)
+    words_japanese = search_japanese(string)
+    
+    results_conversion = PrimeWordList::merge_with_label(@context,
+                                                         words_overall,
+                                                         words_japanese)
+    return PrimeWordList.new(words_compact | results_conversion)
   end
 
   def lookup_prefix (string)
-    # ֤袪ͽ¬
-    expands = PrimeTypeConv::expand(string)
-    query = PrimeQuery.new(expands, nil, :prefix, @context)
-    results = lookup_internal(query)
-    return results
+    words_prefix = search_prefix(string)
+    return PrimeWordList::merge_with_label(@context, words_prefix)
   end
 
   def lookup_exact (string)
-    # ֤褽ͽ¬
-    converted = PrimeTypeConv::convert(string).join()
-    query = PrimeQuery.new([converted, string], nil, :exact, @context)
-    results = lookup_internal(query)
-    return results
+    words_exact = search_exact(string)
+    return PrimeWordList::merge_with_label(@context, words_exact)
   end
 
   def lookup_overall (string)
-    # 1+1=2
-    query = PrimeQuery.new([string], nil, :overall)
-    results = lookup_internal(query)
-    return results
+    words_overall = search_overall(string)
+    return PrimeWordList::merge_with_label(@context, words_overall)
   end
 
   def lookup_expansion (string)
@@ -219,17 +247,134 @@ class Prime
     return false
   end
 
+
   def search (query)
+    query.input.uniq!()
     words_list = @engines.command(:search, query)
     return PrimeWordList::merge(words_list)
   end
 
+  def search_prefix (string)
+    if @cache_prefix_pattern == string then
+      return @cache_prefix
+    end
+
+    # ֤袪ͽ¬
+    expands = PrimeTypeConv::expand(string)
+    query = PrimeQuery.new(expands, nil, :prefix, @context)
+    words = search(query)
+
+    @cache_prefix_pattern = string
+    @cache_prefix = words
+
+    return words
+  end
+
+  def search_exact (string)
+    # ֤褽ͽ¬
+    converted = PrimeTypeConv::convert(string).join()
+    query = PrimeQuery.new([converted, string], nil, :exact, @context)
+    words = search(query)
+    return words
+  end
+  def search_raw (string)
+    ## FIXME: This method is an ad-hoc routine for search_japanese.
+    ## FIXME: <komatsu@taiyaki.org> (2004-02-28)
+    query = PrimeQuery.new([string], nil, :exact, @context)
+    words = search(query)
+    return words
+  end
+
+  def search_overall (string)
+    # 1+1=2, aiueo
+    query = PrimeQuery.new([string], nil, :overall)
+    words_overall = search(query)
+    return words_overall
+  end
+
+  def search_compact (string)
+    if @cache_compact_pattern == string then
+      return @cache_compact
+    end
+    expands = PrimeTypeConv::expand(string)
+    query   = PrimeQuery.new(expands, nil, :prefix, @context)
+    words_compact = search(query)[0,1]
+
+    if words_compact.length > 0 then
+      predict_with_multi_clauses!(words_compact)
+
+      words_compact[1..-1].each {|word|
+        word.score = words_compact[0].score
+      }
+    end
+
+    @cache_compact_pattern = string
+    @cache_compact         = words_compact
+    return words_compact
+  end
+
+  def search_hybrid (string, max = nil)
+    words_prefix   = search_prefix(string)
+    words_japanese = search_japanese(string)
+    words_overall  = search_overall(string)
+
+    num_prefix = 3
+    num_max_prefix = 10
+    if (num_max_prefix - words_japanese.length) > num_prefix then
+      num_prefix = (num_max_prefix - words_japanese.length)
+    end
+    words = PrimeWordList::merge_with_label(@context, 
+                                            words_prefix[0,num_prefix],
+                                            words_overall,
+                                            words_japanese)
+    if max then
+      words = words[0,max]
+    end
+    predict_with_multi_clauses!(words)
+
+    return words
+  end
+
+
   private
   def lookup_internal (query)
+    query.input.uniq!()
     words_list = @engines.command(:search, query)
     return PrimeWordList::merge_with_label(@context, words_list)
   end
 
+  def predict_with_multi_clauses!(results)
+    if results.empty? then
+      string.decrease {|string2|
+        expands = PrimeTypeConv::expand(string2)
+        if expands.length > 2 then
+          next
+        end
+        query = PrimeQuery.new(expands, nil, :exact, @context)
+        word1 = search(query).first
+        
+        if word1.non_nil? then
+          expands = PrimeTypeConv::expand(string[(string2.length)..-1])
+          query = PrimeQuery.new(expands, nil, :context, word1.literal)
+          word2 = search(query).first
+          if word2.non_nil? then
+            results.push(_merge_words(word1, word2))
+            break
+          end
+        end
+      }
+      word = PrimeWordList::merge_with_label(@context, results)[0,1]
+    else
+      word1 = results.first
+      query = PrimeQuery.new([""], nil, :context, word1.literal)
+      word2 = search(query).first
+      if word2.non_nil? then
+        results.push(_merge_words(word1, word2))
+      end
+    end
+    return results
+  end
+
   public
   def Prime::get_prefix (context, literal)
     if PRIME_ENV['style_auto_space'] and context and context != "" then
@@ -253,7 +398,7 @@ class PrimeQuery
   def initialize(input = [], pos = nil, method = :exact, context = nil)
     @input   = input
     @pos     = pos
-    # method = {:prefix, :exact, :literal_prefix, :literal_exact,:overall}
+    # method = {:prefix, :exact, :literal_prefix, :literal_exact,:overall, :context}
     @method  = method 
     @context = context
   end
@@ -313,7 +458,8 @@ class PrimeWordList < Array
     ## FIXME: Isn't it necessary to sort here?
     ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
     result = words.flatten.compact.sort {|word1, word2| 
-      word2.score <=> word1.score
+      (word2.score != word1.score) ? (word2.score <=> word1.score) :
+             (word1.pron.length <=> word2.pron.length)
     }
 
     mark = {}
@@ -340,22 +486,25 @@ class PrimeWordList < Array
     ## FIXME: Isn't it necessary to sort here?
     ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
     result = words.flatten.compact.sort {|word1, word2| 
-      word2.score <=> word1.score
+      (word2.score != word1.score) ? (word2.score <=> word1.score) :
+             (word1.pron.length <=> word2.pron.length)
     }
 
     mark = {}
     merged = PrimeWordList.new
     result.each {|word|
-      if mark[word.label()] then
-        word0 = mark[word.label()]
-        ## FIXME: Make this merging more intelligence.
-        ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
-        word0.category   = (word0.category   or word.category)
-        word0.annotation = (word0.annotation or word.annotation)
-      else
-	mark[word.label()] = word
-        word.prefix = Prime::get_prefix(context, word.literal)
-	merged.push(word)
+      if word.literal != "" then
+        if mark[word.label()] then
+          word0 = mark[word.label()]
+          ## FIXME: Make this merging more intelligence.
+          ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
+          word0.category   = (word0.category   or word.category)
+          word0.annotation = (word0.annotation or word.annotation)
+        else
+          mark[word.label()] = word
+          word.prefix = Prime::get_prefix(context, word.literal)
+          merged.push(word)
+        end
       end
     }
     return merged
