@@ -1,5 +1,5 @@
 # session.rb
-# $Id: session.rb,v 1.1.1.1.2.4 2004/02/08 11:58:07 komatsu Exp $
+# $Id: session.rb,v 1.4 2004/03/25 17:19:49 komatsu Exp $
 #
 # Copyright (C) 2001 Satoru Takabayashi <satoru@namazu.org>
 # Copyright (C) 2002, 2003, 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
@@ -87,6 +87,9 @@ class Session < SessionCore
   end
 
   def execute (line)
+    if line.nil? then
+      return false
+    end
     chunks = line.gsub(/[\r\n]/, "").split("\t")
     if chunks.length > 0 then
       name = chunks.shift.intern
@@ -124,10 +127,20 @@ class SessionPrime < Session
 		"look up PATTERN", 0)
     add_command(:lookup, [:PATTERN],
 		"look up PATTERN", 0)
+    add_command(:lookup_all, [:PATTERN],
+		"look up PATTERN", 0)
+    add_command(:lookup_compact, [:PATTERN],
+		"look up PATTERN and return one or two candidate", 0)
+    add_command(:lookup_compact_all, [:PATTERN],
+		"look up PATTERN and return one or two candidate", 0)
     add_command(:lookup_direct, [:PATTERN],
 		"look up PATTERN for direct typing method", 0)
+    add_command(:lookup_direct_all, [:PATTERN],
+		"look up PATTERN for direct typing method", 0)
     add_command(:lookup_hybrid, [:PATTERN],
 		"look up PATTERN with hybrid matching", 0)
+    add_command(:lookup_hybrid_all, [:PATTERN],
+		"look up PATTERN with hybrid matching", 0)
     add_command(:lookup_prefix, [:PATTERN],
 		"look up PATTERN with prefix matching", 0)
     add_command(:lookup_exact, [:PATTERN],
@@ -196,13 +209,28 @@ class SessionPrime < Session
   def lookup (pattern = "")
     return reply_successful(@prime.lookup(pattern).to_text)
   end
+  def lookup_all (pattern = "")
+    return reply_successful(@prime.lookup_all(pattern).to_text)
+  end
 
+  def lookup_compact (pattern = "")
+    return reply_successful(@prime.lookup_compact(pattern).to_text)
+  end
+  def lookup_compact_all (pattern = "")
+    return reply_successful(@prime.lookup_compact_all(pattern).to_text)
+  end
   def lookup_direct (pattern = "")
     return reply_successful(@prime.lookup_direct(pattern).to_text)
   end
+  def lookup_direct_all (pattern = "")
+    return reply_successful(@prime.lookup_direct_all(pattern).to_text)
+  end
   def lookup_hybrid (pattern = "")
     return reply_successful(@prime.lookup_hybrid(pattern).to_text)
   end
+  def lookup_hybrid_all (pattern = "")
+    return reply_successful(@prime.lookup_hybrid_all(pattern).to_text)
+  end
   def lookup_prefix (pattern = "")
     return reply_successful(@prime.lookup_prefix(pattern).to_text)
   end
